require_relative "pledge_pool"
require_relative 'fundable'

module CrowdFund
  class Project  
    include Fundable

    attr_reader :target_funding_amount, :pledges
    attr_accessor :name, :current_funding_amount

    def initialize(name, current_funding_amount=0, target_funding_amount=1000)
      @name = name
      @current_funding_amount = current_funding_amount
      @target_funding_amount = target_funding_amount
      @pledges = Hash.new(0)
    end

    def funding_needed
      @target_funding_amount - total_funds
    end

    def total_pledges
      @pledges.values.reduce(0) {|sum, value| sum + value}
    end 

    def total_funds
      @current_funding_amount + total_pledges
    end   

    def to_s
      "Project #{@name} has $#{total_funds} in funding towards a goal of $#{@target_funding_amount}."
    end

    def each_pledge
      @pledges.each do |name, amount|
        yield(Pledge.new(name, amount))
      end
    end
  end
end